"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _Lexer_instances, _Lexer_position, _Lexer_expression, _Lexer_chars, _Lexer_current, _Lexer_length, _Lexer_consumeComparatorSigns, _Lexer_consumeEqualSign, _Lexer_consumeIdentifier, _Lexer_consumeNegativeNumber, _Lexer_consumeNumber, _Lexer_consumeSquareBracket, _Lexer_initializeForExpression, _Lexer_next, _Lexer_consumeUntil, _Lexer_consumeLiteral, _Lexer_consumeQuotedIdentifier, _Lexer_consumeRawStringLiteral, _Lexer_matchOrElse;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Lexer = void 0;
const constants_js_1 = require("./constants.js");
const errors_js_1 = require("./errors.js");
/**
 * A lexer for JMESPath expressions.
 *
 * This lexer tokenizes a JMESPath expression into a sequence of tokens.
 */
class Lexer {
    constructor() {
        _Lexer_instances.add(this);
        _Lexer_position.set(this, void 0);
        _Lexer_expression.set(this, void 0);
        _Lexer_chars.set(this, void 0);
        _Lexer_current.set(this, void 0);
        _Lexer_length.set(this, void 0);
        /**
         * Consume a comparator sign.
         *
         * This method is called when the lexer encounters a comparator sign.
         *
         * @param current The current character
         */
        _Lexer_consumeComparatorSigns.set(this, (current) => {
            switch (current) {
                case '<':
                    return __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_matchOrElse).call(this, '=', 'lte', 'lt');
                case '>':
                    return __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_matchOrElse).call(this, '=', 'gte', 'gt');
                case '!':
                    return __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_matchOrElse).call(this, '=', 'ne', 'not');
                case '|':
                    return __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_matchOrElse).call(this, '|', 'or', 'pipe');
                case '&':
                    return __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_matchOrElse).call(this, '&', 'and', 'expref');
                default:
                    return __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeEqualSign).call(this);
            }
        });
    }
    /**
     * Tokenize a JMESPath expression.
     *
     * This method is a generator that yields tokens for the given expression.
     *
     * @param expression The JMESPath expression to tokenize.
     */
    *tokenize(expression) {
        __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_initializeForExpression).call(this, expression);
        while (__classPrivateFieldGet(this, _Lexer_current, "f") !== '' && __classPrivateFieldGet(this, _Lexer_current, "f") !== undefined) {
            if (constants_js_1.SIMPLE_TOKENS.has(__classPrivateFieldGet(this, _Lexer_current, "f"))) {
                yield {
                    // We know that SIMPLE_TOKENS has this.#current as a key because
                    // we checked for that above.
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    type: constants_js_1.SIMPLE_TOKENS.get(__classPrivateFieldGet(this, _Lexer_current, "f")),
                    value: __classPrivateFieldGet(this, _Lexer_current, "f"),
                    start: __classPrivateFieldGet(this, _Lexer_position, "f"),
                    end: __classPrivateFieldGet(this, _Lexer_position, "f") + 1,
                };
                __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
            }
            else if (constants_js_1.START_IDENTIFIER.has(__classPrivateFieldGet(this, _Lexer_current, "f"))) {
                yield __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeIdentifier).call(this);
            }
            else if (constants_js_1.WHITESPACE.has(__classPrivateFieldGet(this, _Lexer_current, "f"))) {
                __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
            }
            else if (__classPrivateFieldGet(this, _Lexer_current, "f") === '[') {
                yield __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeSquareBracket).call(this);
            }
            else if (__classPrivateFieldGet(this, _Lexer_current, "f") === `'`) {
                yield __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeRawStringLiteral).call(this);
            }
            else if (__classPrivateFieldGet(this, _Lexer_current, "f") === '`') {
                yield __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeLiteral).call(this);
            }
            else if (constants_js_1.VALID_NUMBER.has(__classPrivateFieldGet(this, _Lexer_current, "f"))) {
                const start = __classPrivateFieldGet(this, _Lexer_position, "f");
                const buff = __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeNumber).call(this);
                yield {
                    type: 'number',
                    value: parseInt(buff),
                    start: start,
                    end: start + buff.length,
                };
            }
            else if (__classPrivateFieldGet(this, _Lexer_current, "f") === '-') {
                yield __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeNegativeNumber).call(this);
            }
            else if (__classPrivateFieldGet(this, _Lexer_current, "f") === '"') {
                yield __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeQuotedIdentifier).call(this);
            }
            else if (['<', '>', '!', '=', '|', '&'].includes(__classPrivateFieldGet(this, _Lexer_current, "f"))) {
                yield __classPrivateFieldGet(this, _Lexer_consumeComparatorSigns, "f").call(this, __classPrivateFieldGet(this, _Lexer_current, "f"));
            }
            else {
                throw new errors_js_1.LexerError(__classPrivateFieldGet(this, _Lexer_position, "f"), __classPrivateFieldGet(this, _Lexer_current, "f"));
            }
        }
        yield { type: 'eof', value: '', start: __classPrivateFieldGet(this, _Lexer_length, "f"), end: __classPrivateFieldGet(this, _Lexer_length, "f") };
    }
}
exports.Lexer = Lexer;
_Lexer_position = new WeakMap(), _Lexer_expression = new WeakMap(), _Lexer_chars = new WeakMap(), _Lexer_current = new WeakMap(), _Lexer_length = new WeakMap(), _Lexer_consumeComparatorSigns = new WeakMap(), _Lexer_instances = new WeakSet(), _Lexer_consumeEqualSign = function _Lexer_consumeEqualSign() {
    if (__classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this) === '=') {
        __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
        return {
            type: 'eq',
            value: '==',
            start: __classPrivateFieldGet(this, _Lexer_position, "f") - 1,
            end: __classPrivateFieldGet(this, _Lexer_position, "f"),
        };
    }
    else {
        throw new errors_js_1.LexerError(__classPrivateFieldGet(this, _Lexer_position, "f") - 1, '=');
    }
}, _Lexer_consumeIdentifier = function _Lexer_consumeIdentifier() {
    const start = __classPrivateFieldGet(this, _Lexer_position, "f");
    let buff = __classPrivateFieldGet(this, _Lexer_current, "f");
    while (constants_js_1.VALID_IDENTIFIER.has(__classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this))) {
        buff += __classPrivateFieldGet(this, _Lexer_current, "f");
    }
    return {
        type: 'unquoted_identifier',
        value: buff,
        start,
        end: start + buff.length,
    };
}, _Lexer_consumeNegativeNumber = function _Lexer_consumeNegativeNumber() {
    const start = __classPrivateFieldGet(this, _Lexer_position, "f");
    const buff = __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeNumber).call(this);
    if (buff.length > 1) {
        return {
            type: 'number',
            value: parseInt(buff),
            start: start,
            end: start + buff.length,
        };
    }
    else {
        // If the negative sign is not followed by a number, it is an error.
        throw new errors_js_1.LexerError(start, 'Unknown token after "-"');
    }
}, _Lexer_consumeNumber = function _Lexer_consumeNumber() {
    let buff = __classPrivateFieldGet(this, _Lexer_current, "f");
    while (constants_js_1.VALID_NUMBER.has(__classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this))) {
        buff += __classPrivateFieldGet(this, _Lexer_current, "f");
    }
    return buff;
}, _Lexer_consumeSquareBracket = function _Lexer_consumeSquareBracket() {
    const start = __classPrivateFieldGet(this, _Lexer_position, "f");
    const nextChar = __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
    if (nextChar == ']') {
        __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
        return { type: 'flatten', value: '[]', start: start, end: start + 2 };
    }
    else if (nextChar == '?') {
        __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
        return { type: 'filter', value: '[?', start: start, end: start + 2 };
    }
    else {
        return { type: 'lbracket', value: '[', start: start, end: start + 1 };
    }
}, _Lexer_initializeForExpression = function _Lexer_initializeForExpression(expression) {
    if (typeof expression !== 'string') {
        throw new errors_js_1.EmptyExpressionError();
    }
    __classPrivateFieldSet(this, _Lexer_position, 0, "f");
    __classPrivateFieldSet(this, _Lexer_expression, expression, "f");
    __classPrivateFieldSet(this, _Lexer_chars, Array.from(expression), "f");
    __classPrivateFieldSet(this, _Lexer_current, __classPrivateFieldGet(this, _Lexer_chars, "f")[0], "f");
    __classPrivateFieldSet(this, _Lexer_length, __classPrivateFieldGet(this, _Lexer_expression, "f").length, "f");
}, _Lexer_next = function _Lexer_next() {
    if (__classPrivateFieldGet(this, _Lexer_position, "f") === __classPrivateFieldGet(this, _Lexer_length, "f") - 1) {
        __classPrivateFieldSet(this, _Lexer_current, '', "f");
    }
    else {
        __classPrivateFieldSet(this, _Lexer_position, __classPrivateFieldGet(this, _Lexer_position, "f") + 1, "f");
        __classPrivateFieldSet(this, _Lexer_current, __classPrivateFieldGet(this, _Lexer_chars, "f")[__classPrivateFieldGet(this, _Lexer_position, "f")], "f");
    }
    return __classPrivateFieldGet(this, _Lexer_current, "f");
}, _Lexer_consumeUntil = function _Lexer_consumeUntil(delimiter) {
    const start = __classPrivateFieldGet(this, _Lexer_position, "f");
    let buff = '';
    __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
    while (__classPrivateFieldGet(this, _Lexer_current, "f") !== delimiter) {
        if (__classPrivateFieldGet(this, _Lexer_current, "f") === '\\') {
            buff += '\\';
            __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
        }
        if (__classPrivateFieldGet(this, _Lexer_current, "f") === '') {
            // We've reached the end of the expression (EOF) before
            // we found the delimiter. This is an error.
            throw new errors_js_1.LexerError(start, __classPrivateFieldGet(this, _Lexer_expression, "f").substring(start));
        }
        buff += __classPrivateFieldGet(this, _Lexer_current, "f");
        __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
    }
    // Skip the closing delimiter
    __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
    return buff;
}, _Lexer_consumeLiteral = function _Lexer_consumeLiteral() {
    const start = __classPrivateFieldGet(this, _Lexer_position, "f");
    const lexeme = __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeUntil).call(this, '`').replace('\\`', '`');
    try {
        const parsedJson = JSON.parse(lexeme);
        return {
            type: 'literal',
            value: parsedJson,
            start,
            end: __classPrivateFieldGet(this, _Lexer_position, "f") - start,
        };
    }
    catch (error) {
        throw new errors_js_1.LexerError(start, lexeme);
    }
}, _Lexer_consumeQuotedIdentifier = function _Lexer_consumeQuotedIdentifier() {
    const start = __classPrivateFieldGet(this, _Lexer_position, "f");
    const lexeme = '"' + __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeUntil).call(this, '"') + '"';
    const tokenLen = __classPrivateFieldGet(this, _Lexer_position, "f") - start;
    return {
        type: 'quoted_identifier',
        value: JSON.parse(lexeme),
        start,
        end: tokenLen,
    };
}, _Lexer_consumeRawStringLiteral = function _Lexer_consumeRawStringLiteral() {
    const start = __classPrivateFieldGet(this, _Lexer_position, "f");
    const lexeme = __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_consumeUntil).call(this, `'`).replace(`\\'`, `'`);
    const tokenLen = __classPrivateFieldGet(this, _Lexer_position, "f") - start;
    return {
        type: 'literal',
        value: lexeme,
        start,
        end: tokenLen,
    };
}, _Lexer_matchOrElse = function _Lexer_matchOrElse(expected, matchType, elseType) {
    const start = __classPrivateFieldGet(this, _Lexer_position, "f");
    const current = __classPrivateFieldGet(this, _Lexer_current, "f");
    const nextChar = __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
    if (nextChar === expected) {
        __classPrivateFieldGet(this, _Lexer_instances, "m", _Lexer_next).call(this);
        return {
            type: matchType,
            value: current + nextChar,
            start,
            end: start + 2,
        };
    }
    return {
        type: elseType,
        value: current,
        start,
        end: start,
    };
};
