/**
 * The binding powers for the various tokens in the JMESPath grammar.
 *
 * The binding powers are used to determine the order of operations for
 * the parser. The higher the binding power, the more tightly the token
 * binds to its arguments.
 */
declare const BINDING_POWER: {
    readonly eof: 0;
    readonly unquoted_identifier: 0;
    readonly quoted_identifier: 0;
    readonly literal: 0;
    readonly rbracket: 0;
    readonly rparen: 0;
    readonly comma: 0;
    readonly rbrace: 0;
    readonly number: 0;
    readonly current: 0;
    readonly expref: 0;
    readonly colon: 0;
    readonly pipe: 1;
    readonly or: 2;
    readonly and: 3;
    readonly eq: 5;
    readonly gt: 5;
    readonly lt: 5;
    readonly gte: 5;
    readonly lte: 5;
    readonly ne: 5;
    readonly flatten: 9;
    readonly star: 20;
    readonly filter: 21;
    readonly dot: 40;
    readonly not: 45;
    readonly lbrace: 50;
    readonly lbracket: 55;
    readonly lparen: 60;
};
/**
 * The set of ASCII letters and digits allowed in JMESPath identifiers.
 */
declare const START_IDENTIFIER: Set<string>;
/**
 * The set of ASCII letters and digits allowed in JMESPath identifiers.
 */
declare const VALID_IDENTIFIER: Set<string>;
/**
 * The set of ASCII digits allowed in JMESPath identifiers.
 */
declare const VALID_NUMBER: Set<string>;
/**
 * The set of ASCII whitespace characters allowed in JMESPath identifiers.
 */
declare const WHITESPACE: Set<string>;
/**
 * The set of simple tokens in the JMESPath grammar.
 */
declare const SIMPLE_TOKENS: Map<string, keyof typeof BINDING_POWER>;
export { BINDING_POWER, SIMPLE_TOKENS, START_IDENTIFIER, VALID_IDENTIFIER, VALID_NUMBER, WHITESPACE, };
//# sourceMappingURL=constants.d.ts.map